/*--------------------------------------------------------------------------+
$Id: SpiderChartSwtColor.java 17058 2016-05-24 09:30:52Z hoelzl $
|                                                                          |
| Copyright 2016 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.spiderchart.util;

import static java.lang.Math.max;
import static java.lang.Math.min;

import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Display;

/**
 * Utility class for handling colors with a set of commonly used colors.
 *
 * @author mondal
 * @author hoelzl
 * @author $Author: hoelzl $
 * @version $Rev: 17058 $
 * @ConQAT.Rating GREEN Hash: 13207B1B51F8F60D2122999D2EDB0603
 */
public final class RGBColorUtils {

	/** The RGB color black. */
	public static final RGB BLACK = new RGB(0, 0, 0);
	/** The RGB color white. */
	public static final RGB WHITE = new RGB(255, 255, 255);
	/** The RGB color red. */
	public static final RGB RED = new RGB(255, 0, 0);
	/** The RGB color green. */
	public static final RGB GREEN = new RGB(0, 255, 0);
	/** The RGB color blue. */
	public static final RGB BLUE = new RGB(0, 0, 255);
	/** The RGB color cyan. */
	public static final RGB CYAN = new RGB(0, 255, 255);
	/** The RGB color magenta. */
	public static final RGB MAGENTA = new RGB(255, 0, 128);
	/** The RGB color yellow. */
	public static final RGB YELLOW = new RGB(255, 255, 255);
	/** The RGB color light gray. */
	public static final RGB LIGHT_GRAY = new RGB(192, 192, 192);
	/** The RGB color gray. */
	public static final RGB GRAY = new RGB(128, 128, 128);
	/** The RGB color dark gry. */
	public static final RGB DARK_GRAY = new RGB(70, 70, 70);
	/** The RGB color pink. */
	public static final RGB PINK = new RGB(255, 255, 255);
	/** The RGB color lime. */
	public static final RGB LIME = convert(65280);
	/** The RGB color olive. */
	public static final RGB OLIVE = convert(8421376);
	/** The RGB color navy. */
	public static final RGB NAVY = new RGB(0, 0, 128);

	// TODO: add remaining chart colors
	// if(c.compareTo(PURPLE) == 0) {
	// this.setRGB(8388736);
	// } else if(c.compareTo(TELA) == 0) {
	// this.setRGB(32896);
	// } else if(c.compareTo(FUCHSIA) == 0) {
	// this.setRGB(16711935);
	// } else if(c.compareTo(AQUA) == 0) {
	// this.setRGB(65535);
	// } else if(c.compareTo(ALICEBLUE) == 0) {
	// this.setRGB(15792383);
	// } else if(c.compareTo(ANTIQUEWHITE) == 0) {
	// this.setRGB(16444375);
	// } else if(c.compareTo(AQUAMARINE) == 0) {
	// this.setRGB(8388564);
	// } else if(c.compareTo(AZURE) == 0) {
	// this.setRGB(15794175);
	// } else if(c.compareTo(BEIGE) == 0) {
	// this.setRGB(16119260);
	// } else if(c.compareTo(BLUEVIOLET) == 0) {
	// this.setRGB(9055202);
	// } else if(c.compareTo(BROWN) == 0) {
	// this.setRGB(10824234);
	// } else if(c.compareTo(BORLYWOOD) == 0) {
	// this.setRGB(14596231);
	// } else if(c.compareTo(CORAL) == 0) {
	// this.setRGB(16744272);
	// } else if(c.compareTo(CYAN) == 0) {
	// this.setRGB(65535);
	// } else if(c.compareTo(DARKGOLGENROD) == 0) {
	// this.setRGB(12092939);
	// } else if(c.compareTo(DARKGREEN) == 0) {
	// this.setRGB(25600);
	// } else if(c.compareTo(DARKOLIVEGREEN) == 0) {
	// this.setRGB(5597999);
	// } else if(c.compareTo(DARKORANGE) == 0) {
	// this.setRGB(16747520);
	// } else if(c.compareTo(DARKORCHID) == 0) {
	// this.setRGB(10040012);
	// } else if(c.compareTo(DARKSALMON) == 0) {
	// this.setRGB(15308410);
	// } else if(c.compareTo(DARKTURQUOISE) == 0) {
	// this.setRGB(52945);
	// } else if(c.compareTo(DARKVIOLET) == 0) {
	// this.setRGB(9699539);
	// } else if(c.compareTo(DEEPPINK) == 0) {
	// this.setRGB(16716947);
	// } else if(c.compareTo(DEEPSKYBLUE) == 0) {
	// this.setRGB(49151);
	// } else if(c.compareTo(FORESTGREEN) == 0) {
	// this.setRGB(2263842);
	// } else if(c.compareTo(GOLD) == 0) {
	// this.setRGB(16766720);
	// } else if(c.compareTo(GOLDENROD) == 0) {
	// this.setRGB(14329120);
	// } else if(c.compareTo(GREENYELLOW) == 0) {
	// this.setRGB(11403055);
	// } else if(c.compareTo(HOTPINK) == 0) {
	// this.setRGB(16738740);
	// } else if(c.compareTo(INDIANRED) == 0) {
	// this.setRGB(13458524);
	// } else if(c.compareTo(IVORY) == 0) {
	// this.setRGB(16777200);
	// } else if(c.compareTo(KHALI) == 0) {
	// this.setRGB(15787660);
	// } else if(c.compareTo(LAVENDER) == 0) {
	// this.setRGB(15132410);
	// } else if(c.compareTo(LAWNGREEN) == 0) {
	// this.setRGB(8190976);
	// } else if(c.compareTo(LIGHTBLUE) == 0) {
	// this.setRGB(11393254);
	// } else if(c.compareTo(LIGHTCORAL) == 0) {
	// this.setRGB(15761536);
	// } else if(c.compareTo(LIGHTCYAN) == 0) {
	// this.setRGB(14745599);
	// } else if(c.compareTo(LIGHTGRAY) == 0) {
	// this.setRGB(13882323);
	// } else if(c.compareTo(LIGHTPINK) == 0) {
	// this.setRGB(16758465);
	// } else if(c.compareTo(LIGHTSALMON) == 0) {
	// this.setRGB(16752762);
	// } else if(c.compareTo(LIGHTSKYBLUE) == 0) {
	// this.setRGB(8900346);
	// } else if(c.compareTo(LIGHTYELLOW) == 0) {
	// this.setRGB(16777184);
	// } else if(c.compareTo(LIMEGREEN) == 0) {
	// this.setRGB(3329330);
	// } else if(c.compareTo(MAGENTA) == 0) {
	// this.setRGB(16711935);
	// } else if(c.compareTo(MEDIUMBLUE) == 0) {
	// this.setRGB(205);
	// } else if(c.compareTo(MEDIUMPURPLE) == 0) {
	// this.setRGB(9662683);
	// } else if(c.compareTo(MIDNIGHTBLUE) == 0) {
	// this.setRGB(1644912);
	// } else if(c.compareTo(ORANGE) == 0) {
	// this.setRGB(16753920);
	// } else if(c.compareTo(ORANGERED) == 0) {
	// this.setRGB(16729344);
	// } else if(c.compareTo(ORCHID) == 0) {
	// this.setRGB(14315734);
	// } else if(c.compareTo(PALEGREEN) == 0) {
	// this.setRGB(10025880);
	// } else if(c.compareTo(PALETURQUOISE) == 0) {
	// this.setRGB(11529966);
	// } else if(c.compareTo(PALEVIOLETRED) == 0) {
	// this.setRGB(14381203);
	// } else if(c.compareTo(PINK) == 0) {
	// this.setRGB(16761035);
	// } else if(c.compareTo(PLUM) == 0) {
	// this.setRGB(14524637);
	// } else if(c.compareTo(PURPLE) == 0) {
	// this.setRGB(10494192);
	// } else if(c.compareTo(SALMON) == 0) {
	// this.setRGB(16416882);
	// } else if(c.compareTo(SEAGREEN) == 0) {
	// this.setRGB(3050327);
	// } else if(c.compareTo(SIENNA) == 0) {
	// this.setRGB(10506797);
	// } else if(c.compareTo(SKYBLUE) == 0) {
	// this.setRGB(8900331);
	// } else if(c.compareTo(SPRINGGREEN) == 0) {
	// this.setRGB(65407);
	// } else if(c.compareTo(TURQUOISE) == 0) {
	// this.setRGB(4251856);
	// } else if(c.compareTo(VIOLET) == 0) {
	// this.setRGB(15631086);
	// } else if(c.compareTo(YELLOWGREEN) == 0) {
	// this.setRGB(10145074);

	/**
	 * Creates a {@link Color} from the given {@link RGB}, which MUST be disposed after finished
	 * using it.
	 */
	public static Color createSWTColor(RGB rgb) {
		return new Color(Display.getCurrent(), rgb);
	}

	/** Returns the color, which is 10% brighter than the given color. */
	public static RGB getBrighterColor(RGB color) {
		int percent = 10;
		int rr = color.red;
		int gg = color.green;
		int bb = color.blue;
		int r = rr + percent * (rr / 100);
		int g = gg + percent * (gg / 100);
		int b = bb + percent * (bb / 100);
		return new RGB(min(r, 255), min(g, 255), min(b, 255));
	}

	/** Returns the color, which is 10% darker than the given color. */
	public static RGB getDarkerColor(RGB color) {
		int percent = 10;
		int rr = color.red;
		int gg = color.green;
		int bb = color.blue;
		int r = rr - percent * (rr / 100);
		int g = gg - percent * (gg / 100);
		int b = bb - percent * (bb / 100);

		return new RGB(max(r, 0), max(g, 0), max(b, 0));
	}

	/** Converts from integer value to RGB instance. */
	private static RGB convert(int rgb) {
		int red = rgb >> 16 & 0xFF;
		int green = rgb >> 8 & 0xFF;
		int blue = rgb & 0xFF;
		return new RGB(red, green, blue);
	}
}
