/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.spiderchart.widget;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.ControlListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.fortiss.tooling.spiderchart.model.SpiderChart;
import org.fortiss.tooling.spiderchart.style.ChartStyle;

/**
 * Represents a canvas on which chart will be displayed
 *
 * @author mondal
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 13207B1B51F8F60D2122999D2EDB0603
 */
final class SpiderChartSWTCanvas extends Canvas {

	/** The chart data. */
	private final SpiderChart chart;
	/** The chart style. */
	private final ChartStyle style;
	/** The chart widget. */
	private final SpiderChartWidget chartWidget;
	/** The title widget. */
	private final SpiderChartTitleWidget titleWidget;
	/** The legend widget. */
	private final SpiderChartLegendWidget legendWidget;

	/** Constructor */
	public SpiderChartSWTCanvas(Composite parent, SpiderChart chart, ChartStyle style) {
		super(parent, SWT.NONE);
		this.chart = chart;
		this.style = style;
		this.chartWidget = new SpiderChartWidget(chart, style);
		this.titleWidget = new SpiderChartTitleWidget(chart, style);
		this.legendWidget = new SpiderChartLegendWidget(chart, style);

		addPaintListener(e -> SpiderChartSWTCanvas.this.paintChart(e));
		addMouseMoveListener(e -> SpiderChartSWTCanvas.this.mouseMoved(e));

		this.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseDown(MouseEvent e) {
				SpiderChartSWTCanvas.this.mouseClick();
			}
		});

		this.addControlListener(new ControlListener() {
			@Override
			public void controlMoved(ControlEvent e) {
				// Not used
			}

			@Override
			public void controlResized(ControlEvent e) {
				SpiderChartSWTCanvas.this.resizeChart();
			}
		});
	}

	/** Resizes the chart. */
	protected void resizeChart() {
		double charMarginPercent = style.getRelativeMargin();
		Rectangle bounds = getBounds();
		int widthMargin = (int)(bounds.width * charMarginPercent);
		int heightMargin = (int)(bounds.height * charMarginPercent);

		int chartY = 0;
		int widthRemaining = bounds.width;
		int heightRemaining = bounds.height;

		if(style.isShowTitle()) {
			titleWidget.setX(0);
			titleWidget.setY(0);
			titleWidget.setHeight(heightMargin - 1);
			titleWidget.setWidth(bounds.width - 1);
			heightRemaining -= heightMargin;
			chartY = heightMargin;
		}
		if(style.isShowLegend()) {
			if(style.getLegendStyle().isVerticalLayout()) {
				legendWidget.setX(bounds.width - widthMargin);
				legendWidget.setY(bounds.height - heightRemaining);
				legendWidget.setWidth(widthMargin - 1);
				legendWidget.setHeight(heightRemaining - 1);
				widthRemaining -= widthMargin;
			} else {
				legendWidget.setX(0);
				legendWidget.setY(bounds.height - heightMargin);
				legendWidget.setWidth(bounds.width - 1);
				legendWidget.setHeight(heightMargin - 1);
				heightRemaining -= heightMargin;
			}
		}
		chartWidget.setX(0);
		chartWidget.setWidth(widthRemaining);
		chartWidget.setY(chartY);
		chartWidget.setHeight(heightRemaining);
	}

	/** Mouse Click event */
	private void mouseClick() {
		// feature currently disabled
	}

	/** Mouse Moved Event */
	private void mouseMoved(MouseEvent e) {
		// feature currently disabled
	}

	/** Draws the Spider Chart */
	protected void paintChart(PaintEvent e) {
		try {
			if(style.isShowTitle()) {
				titleWidget.draw(e.gc);
			}
			chartWidget.draw(e.gc);
			if(style.isShowLegend()) {
				legendWidget.draw(e.gc);
			}
		} catch(Exception err) {
			err.printStackTrace();
		}
	}

}
