/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.spiderchart.widget;

import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.fortiss.tooling.spiderchart.model.SpiderChart;
import org.fortiss.tooling.spiderchart.style.ChartStyle;
import org.fortiss.tooling.spiderchart.style.FontStyle;

/**
 * Class representing the spider charts title.
 *
 * @author mondal
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: D822418216774A7766E735ECD40429CA
 */
public final class SpiderChartTitleWidget extends SpiderChartWidgetBase {
	/** Constructor */
	public SpiderChartTitleWidget(SpiderChart chart, ChartStyle style) {
		super(chart, style);
	}

	/** Used to draw the title */
	public void draw(GC gc) {
		if(!style.isShowTitle()) {
			return;
		}

		String text = chart.getTitle();
		if(text == null || text.trim().length() == 0) {
			return;
		}

		FontStyle fontStyle = style.getTitleStyle();
		Font oldFont = gc.getFont();
		Font ontheflyFont = fontStyle.createSWTFont();
		gc.setFont(ontheflyFont);

		Color oldColor = gc.getForeground();
		Color color = fontStyle.createSWTColor();
		gc.setForeground(color);

		Point extent = gc.textExtent(text);
		int x = getX() + getWidth() / 2 - extent.x / 2;
		int y = getY() + getHeight() / 2 - extent.y / 2;
		gc.drawString(text, x, y, true);

		gc.setForeground(oldColor);
		color.dispose();
		gc.setFont(oldFont);
		ontheflyFont.dispose();
	}
}
