/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.spiderchart.widget;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.ControlListener;
import org.eclipse.swt.widgets.Composite;
import org.fortiss.tooling.spiderchart.model.SpiderChart;
import org.fortiss.tooling.spiderchart.style.ChartStyle;

/**
 * Represents a viewer on the canvas to display the spider chart.
 *
 * @author mondal
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 13207B1B51F8F60D2122999D2EDB0603
 */
public final class SpiderChartViewer extends Composite {

	/** the actual canvas to be used */
	private SpiderChartSWTCanvas canvas = null;

	/** Constructor */
	public SpiderChartViewer(Composite parent, SpiderChart chart, ChartStyle style) {
		super(parent, SWT.NONE);

		canvas = new SpiderChartSWTCanvas(this, chart, style);

		this.addControlListener(new ControlListener() {
			@Override
			public void controlMoved(ControlEvent e) {
				// Not used
			}

			@Override
			public void controlResized(ControlEvent e) {
				setCanvasSize();
			}
		});
	}

	/** getter for the canvas in use */
	public SpiderChartSWTCanvas getCanvas() {
		return this.canvas;
	}

	/** {@inheritDoc} */
	@Override
	public boolean setFocus() {
		return canvas.setFocus();
	}

	/** Redraws the chart */
	public void redrawChart() {
		canvas.redraw();
	}

	/** Sets the size of the canvas. */
	private void setCanvasSize() {
		canvas.setLocation(0, 0);
		canvas.setSize(this.getSize().x, this.getSize().y);
	}

	/** {@inheritDoc} */
	@Override
	public void dispose() {
		canvas.dispose();
		super.dispose();
	}
}
